//RNN v1.6
//    Copyright (C) 2010  Roberto J. López-Sastre (robertoj.lopez@uah.es)
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
#ifndef AGGCLUSTERING_H
#define AGGCLUSTERING_H

//Includes
#include <iostream>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <vnl/vnl_vector.h>
#include <vnl/vnl_sym_matrix.h>
#include <vnl/vnl_random.h>
#include <vcl_vector.h>  
#include <vcl_deque.h>
#include <vcl_fstream.h>
#include <sys/time.h>

//MACRO
#define TIME_THIS(X,Y) \
{ \
    struct timeval t_ini, t_fin; \
    gettimeofday(&t_ini, NULL); \
    X; \
    gettimeofday(&t_fin, NULL); \
    Y=timeval_diff(&t_fin, &t_ini); \
}

//Structs
// reuturns "a - b" in seconds
double timeval_diff(struct timeval *a, struct timeval *b)
{
  return
    (double)(a->tv_sec + (double)a->tv_usec/1000000) -
    (double)(b->tv_sec + (double)b->tv_usec/1000000);
}

static struct option long_options[]=
{
  {"help",0,NULL,0},
  {"data",1,NULL,1},
  {"dim",1,NULL,2},
  {"n",1,NULL,3},
  {"out",1,NULL,4},
  {"thres",1,NULL,5},
  {NULL,0,0,0}
};

char * data_file_path = NULL;
char * out_file_path = NULL;
unsigned int dim;
unsigned int n;
double thres;    //threshold for agglomerative clustering
unsigned int nc; //num clusters
using namespace std;
unsigned long ndist;

//Types
typedef struct scluster{
  vnl_vector<double> centroid; //centroid   
  vcl_deque<unsigned int> data_index; //index of vectors of this cluster
  double cvar; //cluster variance
}cluster;

//Functions

unsigned int agg_clustering_rnn(vcl_deque < vnl_vector<double> > &X, vcl_vector <unsigned int> &labels, double agg_thres,vcl_vector< vcl_vector<double> > &cluster_centre );
void get_nn(cluster *V,vcl_deque <cluster> &R,double &sim,unsigned int &index);
void agglomerate_clusters(cluster *C1, cluster *C2);
int get_options(int argc,char *argv[]);
int read_data(vcl_deque < vnl_vector<double> >& data);
int save_data(vcl_vector <unsigned int> &IDs,vcl_vector< vcl_vector<double> > &centres,float time,unsigned long num_distances);

#endif
